//
//  BoardTest.m
//  tictactoe
//
//  Created by VENKAT SUBRAMANIAM on 11/30/10.
//  Copyright 2010 Agile Developer, Inc. All rights reserved.
//

#import "BoardTest.h"
#import "Board.h"

@implementation BoardTest

@synthesize board;

-(void) setUp {
	self.board = [[Board alloc] init];
}

-(void) testCanary {
	STAssertTrue(TRUE, @"Canary test should pass");
}

-(void) testCreateBoard {
	STAssertFalse([self.board isGameOver], @"Game should not be over when created");
}

-(void) testSetFirstPlayer {
	[board setFirstPlayerPegIsX: TRUE];
	STAssertTrue([board isFirstPlayerPegX], @"first player should be X here");
}

-(void) testChangeFirstPlayer {
	[board setFirstPlayerPegIsX: TRUE];
	STAssertTrue([board isFirstPlayerPegX], @"first player should be X here");
	[board setFirstPlayerPegIsX: FALSE];
	STAssertFalse([board isFirstPlayerPegX], @"first player should not be X here");
}

-(void) testPlaceFirstPeg {
	[board setFirstPlayerPegIsX: TRUE];
	[board placePegAtRow: 1 column: 2];
	STAssertTrue([board isPegXAtRow: 1 column:2], @"Peg should be X at position");
}

-(void) testChangeFirstPlayerAfterPlacingAPeg {
	[board setFirstPlayerPegIsX: TRUE];
	[board placePegAtRow: 1 column: 2];
	STAssertThrowsSpecific([board setFirstPlayerPegIsX: TRUE], NSException, @"Expected exception for changing first player after placing a peg");
}

-(void) testPlacePegWithoutSettingFirstPlayer {
	STAssertThrowsSpecific([board placePegAtRow: 1 column:2], NSException, @"Expected exception for placing peg without setting first player");	
}

-(void) testPlaceSecondPeg {
	[board setFirstPlayerPegIsX: TRUE];
	[board placePegAtRow: 1 column: 2];
	[board placePegAtRow: 2 column: 2];
	STAssertTrue([board isPegXAtRow: 1 column: 2], @"Peg should be X at position");
	STAssertFalse([board isPegXAtRow: 2 column: 2], @"Peg should not be X at position");
}

-(void) testPlacePegAtOccupiedPosition {
	[board setFirstPlayerPegIsX: TRUE];
	[board placePegAtRow: 1 column: 2];
	STAssertThrowsSpecific([board placePegAtRow: 1 column: 2],
						   NSException, @"Expected exception for placing peg at occupied position");
}

-(void) testGetPegAtEmptyPosition {
	STAssertThrowsSpecific([board isPegXAtRow: 1 column: 2],
						   NSException, @"position is empty, expected exception");
}

-(void) testPlacePegOutOfRowRange {
	STAssertThrowsSpecific([board placePegAtRow: 4 column: 1],
						   NSException, @"Out of row range, expected exception");
}

-(void) testPlacePegOutOfColumnRange {
	STAssertThrowsSpecific([board placePegAtRow: 1 column: 5],
						   NSException, @"Out of column range, expected exception");
}

-(void) testRowWin {
	[board setFirstPlayerPegIsX: TRUE];
	[board placePegAtRow: 0 column: 0];
	[board placePegAtRow: 1 column: 1];
	[board placePegAtRow: 0 column: 1];
	[board placePegAtRow: 2 column: 2];
	[board placePegAtRow: 0 column: 2];
	STAssertTrue([board isGameOver], @"Should be over");
}

-(void) testColumnWin {
	[board setFirstPlayerPegIsX: FALSE];
	[board placePegAtRow: 0 column: 1];
	[board placePegAtRow: 1 column: 2];
	[board placePegAtRow: 1 column: 1];
	[board placePegAtRow: 2 column: 2];
	[board placePegAtRow: 2 column: 1];
	STAssertTrue([board isGameOver], @"Should be over");
}

-(void) testLeftDiagonalWin {
	[board setFirstPlayerPegIsX: FALSE];
	[board placePegAtRow: 0 column: 0];
	[board placePegAtRow: 2 column: 0];
	[board placePegAtRow: 2 column: 2];
	[board placePegAtRow: 1 column: 1];
	[board placePegAtRow: 0 column: 1];
	[board placePegAtRow: 0 column: 2];
	STAssertTrue([board isGameOver], @"Should be over");
}

-(void) testRightDiagonalWin {
	[board setFirstPlayerPegIsX: FALSE];
	[board placePegAtRow: 0 column: 1];
	[board placePegAtRow: 0 column: 0];
	[board placePegAtRow: 0 column: 2];
	[board placePegAtRow: 1 column: 1];
	[board placePegAtRow: 1 column: 2];
	[board placePegAtRow: 2 column: 2];
	STAssertTrue([board isGameOver], @"Should be over");
}

-(void) testFindWhosTheWinner {
	[board setFirstPlayerPegIsX: TRUE];
	[board placePegAtRow: 0 column: 1];
	[board placePegAtRow: 0 column: 0];
	[board placePegAtRow: 0 column: 2];
	[board placePegAtRow: 1 column: 1];
	[board placePegAtRow: 1 column: 2];
	[board placePegAtRow: 2 column: 2];
	STAssertTrue([board isNextPegX], @"Winner should not be X");
}

@end
